(* $Id: debug.ml,v 1.6 2007/09/14 12:21:47 heinz Exp $
 *
 * AUS - the advanced update system
 * Copyright (C) 2004-2005  Heinz Deinhart
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)

let debug_message_level = ref 0
let argv0 = Sys.argv.(0)

let set_debug_level l =
  debug_message_level := l

let inc_debug_level () =
  debug_message_level := !debug_message_level + 1

let debug_level () =
  !debug_message_level

let print lvl str =
  if lvl <= !debug_message_level then begin
    flush_all ();
    Printf.fprintf stderr "%s %i: %s\n" argv0 lvl str;
    flush stderr
  end

(* these dont really belong into Debug and should be moved out to
 * somewhere else (fixme)
 *)
let verbose_level = ref 0

let vprint ?(lvl=2) str =
  if lvl <= !verbose_level then begin
    print_string (str ^ "\n");
    flush stdout
  end

(* small message support *)

let vsmall_level = ref 0
let vsmall_history = ref []

let vsmallmsg msg =
  if !verbose_level != 1 then
    max_int
  else begin
    print_string msg;
    flush stdout;
    incr vsmall_level;
    vsmall_history := (!vsmall_level, msg) :: !vsmall_history;
    print 190 (Printf.sprintf "VSMALLMSG installed msg '%s', lvl=%i" msg
		  (!vsmall_level - 1));
    !vsmall_level - 1
  end
    
let rec vsmalldel lvl =
  let print_backspaces str =
    let l = String.length str
    in
      if l > 0 then begin
        for _ = 1 to l do
          print_char '\b'
        done;
        for _ = 1 to l do
          print_char ' '
        done;
        for _ = 1 to l do
          print_char '\b'
        done;
        flush stdout
      end
  in
    match !vsmall_history with
      | (l, m) :: rest when l > lvl ->
	  vsmall_history := rest;
	  decr vsmall_level;
	  print_backspaces m;
	  print 190 (Printf.sprintf "VSMALLMSG killed msg '%s', newlvl=%i" m
			(!vsmall_level));
	  vsmalldel lvl
      | _ -> ()
