(* $Id: job.ml,v 1.3 2016/01/08 14:25:38 heinz Exp $
 *
 * AUS - the advanced update system
 * Copyright (C) 2004-2005  Heinz Deinhart
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)

exception Internal_error of string * string list
  (** raised on an internal fatal error *)

exception Died_unexpected of string * string list
  (** raised when job dies unexpected, stdout&err output is provided *)

let launch cmd args line_callback =
  let zero = Unix.openfile "/dev/zero" [ Unix.O_RDWR ] 0
  and output, output2 = Unix.pipe ()
  and stdouterr = ref []
  in
    Unix.set_close_on_exec output;
    let pid = Unix.create_process cmd args zero output2 output2
    in let ch = Unix.in_channel_of_descr output
    in
      Unix.close output2;
      try
	while true do
	  let l = input_line ch
	  in
	    stdouterr := l :: !stdouterr;
	    ignore (line_callback l)
	done;
	raise End_of_file
      with
	  End_of_file ->
	    close_in ch;
	    match Unix.waitpid [] pid with
		rpid, Unix.WEXITED x when rpid == pid ->
		  x, !stdouterr
	      | rpid, Unix.WSIGNALED sn when rpid == pid ->
		  raise (Died_unexpected 
			   ((Printf.sprintf "WSIGNLED(%i)" sn), !stdouterr))
	      | rpid, Unix.WSTOPPED sn when rpid == pid ->
		  raise (Died_unexpected 
			   ((Printf.sprintf "WSTOPPED(%i)" sn), !stdouterr))
	      | unknownpid, Unix.WEXITED rc ->
		  let msg = 
		    Printf.sprintf "WEXITED: pid=%i (expected %i), exitcode=%i"
		      unknownpid pid rc
		  in
		  raise (Internal_error (msg, !stdouterr))
	      | unknownpid, Unix.WSIGNALED sn ->
		  let msg = 
		    Printf.sprintf "WSIGNALED: pid=%i (expected %i), signal=%i"
		      unknownpid pid sn
		  in
		  raise (Internal_error (msg, !stdouterr))
	      | unknownpid, Unix.WSTOPPED sn ->
		  let msg = 
		    Printf.sprintf "WSTOPPED: pid=%i (expected %i), signal=%i"
		      unknownpid pid sn
		  in
		  raise (Internal_error (msg, !stdouterr))
