(* $Id: pkgsys.ml,v 1.10 2007/09/14 12:21:47 heinz Exp $
 *
 * AUS - the advanced update system
 * Copyright (C) 2004-2005  Heinz Deinhart
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)

(** Raised on configuration failure *)
exception Configuration_error of string

(** Raised on runtime failure, takes error msg and job output lines *)
exception Runtime_error of string * string list

(** base class for all implementations *)
class virtual packageSystem = object
  (** called once before usage *)
  method virtual init : unit -> unit

  (** parse configuration string *)
  method virtual configure : string list -> unit

  (** Additional include files to read by aus.
      This enables some default group files to be used for some repositories
      and this method is needed since they depend on the configure
      implementation.
  *)
  method virtual additional_includes : unit -> string list

  (** activate configuration *)
  method virtual activate_configuration : unit -> unit

  (** fetch metadata from repositories *)
  method virtual update_metadata:
    ?progress_callback:(float -> string -> unit) -> unit -> unit

  (** upgrade installed packages *)
  method virtual upgrade:
    ?progress_callback:(float -> string -> unit) -> unit -> unit

  (** install some packages *)
  method virtual install: ?progress_callback:(float -> string -> unit) ->
    (string * string option) list -> unit

  (** remove some packages *)
  method virtual remove: string list -> unit

  (** clean packages *)
  method virtual clean: unit -> unit

  (** clean packages and meta data *)
  method virtual distclean: unit -> unit
end
