(* $Id: pkgsysdebug.ml,v 1.8 2007/09/14 12:21:47 heinz Exp $
 *
 * AUS - the advanced update system
 * Copyright (C) 2004-2005  Heinz Deinhart
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)

open Printf

open Pkgsys
open Util

let whitespace_regexp = Str.regexp "[ \t]+"

let say msg = fprintf stderr "PkgSysDebug: %s\n" msg

let string_of_strings strs =
  List.fold_right (fun astr strs -> astr ^ "," ^ strs) strs ""

class pkgSysDebug = object
  inherit packageSystem

  val mutable repo_urls = []

  method init () = repo_urls <- []

  (* even the debug pkgsys must honor the repository location to enable
     parsing of default group files
  *)
  method additional_includes () =
    List.rev (List.map (fun x -> x ^ "/ausgroups.conf") repo_urls)
  method configure msgs =
    let cfg_msg msg =
      match Str.split whitespace_regexp msg with
	| "repository" :: url :: [] -> repo_urls <- url :: repo_urls
	| _ -> ()
    in
      say ("configure called with '" ^ (string_of_strings msgs) ^ "'");
      List.iter cfg_msg msgs
  method activate_configuration () = say "activate configuration called"
  method update_metadata ?(progress_callback=(fun _ _ -> ())) () =
    say "activate metadata called"
  method upgrade ?progress_callback () = say "upgrade called"
  method install ?progress_callback nas =
    say ("install called for '" ^ (string_of_name_archs nas) ^ "'")
  method remove _ (* pkgs *) = say "remove called"
  method clean () = say "clean called"
  method distclean () = say "distclean called"
end
