(* $Id: util.ml,v 1.3 2007/08/30 15:35:23 heinz Exp $
 *
 * AUS - the advanced update system
 * Copyright (C) 2004-2005  Heinz Deinhart
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)

let slash_regexp = Str.regexp "[/]"

let compare_case_insensitive str1 str2 =
  compare (String.lowercase_ascii str1) (String.lowercase_ascii str2)
                 
let name_arch_compare na1 na2 =
  match na1, na2 with
    | (n1, a1), (n2, a2) when (compare_case_insensitive n1 n2) = 0 ->
	compare a1 a2
    | (n1, _), (n2, _) -> compare_case_insensitive n1 n2

let name_arch_from_string str =
  match Str.bounded_split slash_regexp str 2 with
    | [n] -> n, None
    | [n; a] -> n, Some a
    | _ -> raise (Invalid_argument ("illegal package name '" ^ str ^ "'"))

let string_of_name_arch ?(sep="/") = function
  | n, None -> n
  | n, Some a -> n ^ sep ^ a

let string_of_name_archs nas =
  List.fold_right
    (fun na strs -> (string_of_name_arch na) ^ "," ^ strs) nas ""

let get_my_hostname () =
  try
    let ich = Unix.open_process_in "hostname -f"
    in
      try
	let hn = input_line ich
	in
	  begin
	    match Unix.close_process_in ich with
	      Unix.WEXITED 0 -> ()
	      | Unix.WEXITED i ->
		  Debug.print 1 ("get_my_hostname: WEXITED " ^ (string_of_int i));
		  raise Not_found;
	      | _ ->
		  Debug.print 1 "get_my_hostname: !WEXITED";
		  raise Not_found;
	  end;
	  Debug.print 3 ("my hostname is '"^hn^"'");
	  hn
      with
	  End_of_file ->
	    Debug.print 1 "get_my_hostname: got an eof from hostname -f";
	    raise Not_found
  with
      Not_found ->
	let hn = Sys.getenv "HOSTNAME"
	in
	  Debug.print 3 ("my hostname is '"^hn^"'");
	  hn
